open Ast

let rec string_of_expr = function
  | Id(s) -> s
  | Assign(var, param, e) -> ""
  | Call(f, el) ->
    f ^ "(" ^ String.concat ", " (List.map string_of_expr el) ^ ")"
  
let rec string_of_objectexpr = function
  Literal(s) -> s
  | Noexpr -> ""
  | Str(s) -> s
  | Binop(e1, o, e2) ->
      string_of_objectexpr e1 ^ " " ^
      (match o with
	    Add -> "+" | Sub -> "-" | Mult -> "*" | Div -> "/"
      | Equal -> "==" | Neq -> "!="
      | Less -> "<" | Leq -> "<=" | Greater -> ">" | Geq -> ">=" 
      | Combine -> "++" ) ^ " " ^
      string_of_objectexpr e2
  | Access(var, param)-> var ^ "@" ^ param
  | AssignToObject(v, param, e) -> v ^ "@" ^ param ^ "=" ^ string_of_objectexpr e
let rec string_of_stmt = function
    Block(stmts) ->
      "{\n" ^ String.concat "" (List.map string_of_stmt stmts) ^ "}\n"
  | Expr(expr) -> string_of_expr expr ^ ";\n";
  | If(e, s, Block([])) -> "if (" ^ string_of_objectexpr e ^ ")\n" ^ string_of_stmt s
  | If(e, s1, s2) ->  "if (" ^ string_of_objectexpr e ^ ")\n" ^
      string_of_stmt s1 ^ "else\n" ^ string_of_stmt s2
  | For(e1, e2, e3, s) ->
      "for (" ^ string_of_objectexpr e1  ^ " ; " ^ string_of_objectexpr e2 ^ " ; " ^
      string_of_objectexpr e3  ^ ") " ^ string_of_stmt s
  | ToScreen(s) -> "toscreen (" ^ string_of_objectexpr s ^ " )\n"
      
let string_of_vdecl id = "int " ^ id ^ ";\n"

let list_of_vdecl vars = 
""

let string_of_fdecl fdecl =
  fdecl.fname ^ "(" ^ String.concat ", " fdecl.formals ^ ")\n{\n" ^
  String.concat "" (List.map list_of_vdecl fdecl.locals) ^
  String.concat "" (List.map string_of_stmt fdecl.body) ^
  "}\n"

let string_of_program (vars, funcs) =
  String.concat "" (List.map list_of_vdecl vars) ^ "\n" ^
  String.concat "\n" (List.map string_of_fdecl funcs)    